﻿using System.Windows;
using System.Windows.Controls;

namespace TestProject
{
    /// <summary>
    /// 애니메이션 스크롤 뷰어
    /// </summary>
    public class AnimationScrollViewer : ScrollViewer
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 현재 수평 오프셋 속성 - CurrentHorizontalOffsetProperty

        /// <summary>
        /// 현재 수평 오프셋 속성
        /// </summary>
        public static DependencyProperty CurrentHorizontalOffsetProperty = DependencyProperty.Register
        (
            "CurrentHorizontalOffsetOffset",
            typeof(double),
            typeof(AnimationScrollViewer),
            new PropertyMetadata(new PropertyChangedCallback(CurrentHorizontalOffsetPropertyChangedCallback))
        );

        #endregion
        #region 현재 수직 오프셋 속성 - CurrentVerticalOffsetProperty

        /// <summary>
        /// 현재 수직 오프셋 속성
        /// </summary>
        public static DependencyProperty CurrentVerticalOffsetProperty = DependencyProperty.Register
        (
            "CurrentVerticalOffset",
            typeof(double),
            typeof(AnimationScrollViewer),
            new PropertyMetadata(new PropertyChangedCallback(CurrentVerticalOffsetPropertyChangedCallBack))
        );

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 현재 수평 오프셋 - CurrentHorizontalOffset

        /// <summary>
        /// 현재 수평 오프셋
        /// </summary>
        public double CurrentHorizontalOffset
        {
            get
            {
                return (double)GetValue(CurrentHorizontalOffsetProperty);
            }
            set
            {
                SetValue(CurrentHorizontalOffsetProperty, value);
            }
        }

        #endregion
        #region 현재 수직 오프셋 - CurrentVerticalOffset

        /// <summary>
        /// 현재 수직 오프셋
        /// </summary>
        public double CurrentVerticalOffset
        {
            get
            {
                return (double)GetValue(CurrentVerticalOffsetProperty);
            }
            set
            {
                SetValue(CurrentVerticalOffsetProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 현재 수평 오프셋 속성 변경시 콜백 처리하기 - CurrentHorizontalOffsetPropertyChangedCallback(d, e)

        /// <summary>
        /// 현재 수평 오프셋 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void CurrentHorizontalOffsetPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AnimationScrollViewer viewer = d as AnimationScrollViewer;

            viewer.ScrollToHorizontalOffset((double)e.NewValue);
        }

        #endregion
        #region 현재 수직 오프셋 속성 변경시 콜백 처리하기 - CurrentVerticalOffsetPropertyChangedCallBack(d, e)

        /// <summary>
        /// 현재 수직 오프셋 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void CurrentVerticalOffsetPropertyChangedCallBack(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AnimationScrollViewer viewer = d as AnimationScrollViewer;

            viewer.ScrollToVerticalOffset((double)e.NewValue);
        }

        #endregion
    }
}