﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 항목 카운트
        /// </summary>
        private const int ITEM_COUNT = 50;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            for(int i = 0; i < ITEM_COUNT; i++)
            {
                Button button = CreateButton(i);

                ListViewItem item = new ListViewItem();

                item.Tag     = button;
                item.Content = "버튼 " + i.ToString();

                this.targetComboBox.Items.Add(item);

                this.contentStackPanel.Children.Add(button);

                this.animateButton.Click += animateButton_Click;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 애니메이션 시작 버튼 클릭시 처리하기 - animateButton_Click(sender, e)

        /// <summary>
        /// 애니메이션 시작 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void animateButton_Click(object sender, RoutedEventArgs e)
        {
            Button button = ((ListViewItem)this.targetComboBox.SelectedItem).Tag as Button;

            Point relativePoint = button.TransformToAncestor(this.contentStackPanel).Transform(new Point(0, 0));

            ScrollToPosition(relativePoint.X, relativePoint.Y);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 버튼 생성하기 - CreateButton(index)

        /// <summary>
        /// 버튼 생성하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>버튼</returns>
        private Button CreateButton(int index)
        {
            Button button = new Button();

            button.Margin  = new Thickness(5);
            button.Height  = 30;
            button.Content = "버튼 " + index.ToString();

            return button;
        }

        #endregion
        #region 위치까지 스크롤하기 - ScrollToPosition(x, y)

        /// <summary>
        /// 위치까지 스크롤하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        private void ScrollToPosition(double x, double y)
        {
            DoubleAnimation xAnimation = new DoubleAnimation();

            xAnimation.From = this.animationScrollViewer.HorizontalOffset;
            xAnimation.To = x;
            xAnimation.DecelerationRatio = 0.2;
            xAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(1000));

            DoubleAnimation yAnimation = new DoubleAnimation();

            yAnimation.From              = this.animationScrollViewer.VerticalOffset;
            yAnimation.To                = y;
            yAnimation.DecelerationRatio = 0.2;
            yAnimation.Duration          = new Duration(TimeSpan.FromMilliseconds(500));

            Storyboard storyboard = new Storyboard();

            storyboard.Children.Add(xAnimation);
            storyboard.Children.Add(yAnimation);

            Storyboard.SetTarget(xAnimation, this.animationScrollViewer);
            Storyboard.SetTargetProperty(xAnimation, new PropertyPath(AnimationScrollViewer.CurrentHorizontalOffsetProperty));

            Storyboard.SetTarget(yAnimation, this.animationScrollViewer);
            Storyboard.SetTargetProperty(yAnimation, new PropertyPath(AnimationScrollViewer.CurrentVerticalOffsetProperty));

            storyboard.Begin();
        }

        #endregion
    }
}